@@ -1,6 +1,6 @@
  /* ***** BEGIN LICENSE BLOCK *****
  *
- * $Id: encmain.cpp,v 1.28 2005/05/20 14:01:26 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+ * $Id: encmain.cpp,v 1.37 2005/12/01 14:27:47 asuraparaju Exp $ $Name: Dirac_0_5_4 $
  *
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -38,6 +38,7 @@
  * or the LGPL.
  * ***** END LICENSE BLOCK ***** */
 
+#include <time.h>
 #include <iostream>
 #include <limits>
 #include <fstream>
@@ -61,37 +62,39 @@ static void display_help()
 {
     cout << "\nDIRAC wavelet video coder.";
     cout << "\n";
-    cout << "\nUsage: progname -<flag1> [<flag_val>] ... <input1> <input2> ...";
+    cout << "\nUsage: progname -<flag1> [<flag1_val>] ... <input> <output>";
     cout << "\nIn case of multiple assignment to the same parameter, the last holds.";
     cout << "\n";
-    cout << "\nName    Type   I/O Default Value Description";
-    cout << "\n====    ====   === ============= ===========                                       ";
-    cout << "\ninput   string  I  [ required ]  Input file name";
-    cout << "\noutput  string  I  [ required ]  Output file name";
-    cout << "\nCIF     bool    I  true          Use CIF compression presets";
-    cout << "\nHD720   bool    I  false         Use HD-720 compression presets";
-    cout << "\nHD1080  bool    I  false         Use HD-1080 compression presets";
-    cout << "\nSD576   bool    I  false         Use SD-576 compression presets";
-    cout << "\nx       ulong   I  Preset        Width of frame";
-    cout << "\ny       ulong   I  Preset        Length of frame";
-    cout << "\ncformat ulong   I  Yonly         Chroma format";
-    cout << "\nfr     ulong   I  Preset        Frame rate(s) (e.n or e/n format)";
-    cout << "\nstart   ulong   I  0UL           Frame number to start encoding from";
-    cout << "\nstop    ulong   I  EOF           Frame number after which encoding finishes";
-    cout << "\nL1_sep  ulong   I  0UL           Separation of L1 frames";
-    cout << "\nnum_L1  ulong   I  0UL           Number of L1 frames";
-    cout << "\nxblen   ulong   I  0UL           Overlapping block horizontal length";
-    cout << "\nyblen   ulong   I  0UL           Overlapping block vertical length";
-    cout << "\nxbsep   ulong   I  0UL           Overlapping block horizontal separation";
-    cout << "\nybsep   ulong   I  0UL           Overlapping block vertical separation";
-    cout << "\ncpd     ulong   I  0UL           Perceptual weighting - vertical cycles per degree";
-    cout << "\nqf      float   I  0.0F          Overall quality factor (0.0 - 10.0)";
-    cout << "\nverbose bool    I  false         Verbose mode";
-    cout << "\nnolocal bool    I  false         Do no write diagnostics and locally decoded output";
+    cout << "\nName     Type    Default Value Description";
+    cout << "\n====     ====    ============= ===========                                       ";
+    cout << "\ninput    string  [ required ]  Input file name";
+    cout << "\noutput   string  [ required ]  Output file name";
+    cout << "\nCIF      bool    true          Use CIF compression presets";
+    cout << "\nHD720    bool    false         Use HD-720 compression presets";
+    cout << "\nHD1080   bool    false         Use HD-1080 compression presets";
+    cout << "\nSD576    bool    false         Use SD-576 compression presets";
+    cout << "\nwidth    ulong   Preset        Width of frame";
+    cout << "\nheight   ulong   Preset        Length of frame";
+    cout << "\ncformat  ulong   Yonly         Chroma format 0=Yonly 1=422 2=444 3=420 4=411";
+    cout << "\nfr       ulong   Preset        Frame rate(s) (e.n or e/n format)";
+    cout << "\nstart    ulong   0UL           Frame number to start encoding from";
+    cout << "\nstop     ulong   EOF           Frame number after which encoding finishes";
+    cout << "\nL1_sep   ulong   0UL           Separation of L1 frames";
+    cout << "\nnum_L1   ulong   0UL           Number of L1 frames";
+    cout << "\nxblen    ulong   0UL           Overlapping block horizontal length";
+    cout << "\nyblen    ulong   0UL           Overlapping block vertical length";
+    cout << "\nxbsep    ulong   0UL           Overlapping block horizontal separation";
+    cout << "\nybsep    ulong   0UL           Overlapping block vertical separation";
+    cout << "\ncpd      ulong   0UL           Perceptual weighting - vertical cycles per deg.";
+    cout << "\nqf       float   0.0F          Overall quality factor (>0, typically: 7=medium, 9=high)";
+    cout << "\nlossless bool    false         Lossless coding (overrides qf)";
+    cout << "\nverbose  bool    false         Verbose mode";
+    cout << "\nlocal    bool    false         Write diagnostics & locally decoded video";
     cout << endl;
 }
 
 
+
 bool WritePicData (std::ofstream &fdata, dirac_encoder_t *encoder)
 {
     dirac_seqparams_t &sparams = encoder->enc_ctx.seq_params;
@@ -364,14 +367,65 @@ int GetFrameBufferSize (const dirac_encoder_context_t &enc_ctx)
     }
     return size;
 }
+const char *chroma2string (dirac_chroma_t chroma)
+{
+    switch (chroma)
+    {
+    case Yonly:
+        return "Y_ONLY";
+
+    case format422:
+        return "4:2:2";
+
+    case format444:
+        return "4:4:4";
+
+    case format420:
+        return "4:2:0";
+
+    case format411:
+        return "4:1:1";
+
+    default:
+        break;
+    }
+
+    return "Unknown";
+}
+
+void display_codec_params(dirac_encoder_context_t &enc_ctx)
+{
+    std::cout << "Sequence parameters : " << std::endl;
+    std::cout << "\theight=" << enc_ctx.seq_params.height;
+    std::cout << " width=" << enc_ctx.seq_params.width << std::endl;
+    std::cout << "\tchroma=" << chroma2string(enc_ctx.seq_params.chroma) << std::endl;
+    std::cout << "\tframe rate=" << enc_ctx.seq_params.frame_rate.numerator;
+    std::cout << "/" << enc_ctx.seq_params.frame_rate.denominator << std::endl;
+    std::cout << "Encoder parameters : " << std::endl;
+    std::cout << "\tquality factor=" << enc_ctx.enc_params.qf << std::endl;
+    std::cout << "\tGOP parameters : num_L1="  << enc_ctx.enc_params.num_L1;
+    std::cout << " L1_sep=" << enc_ctx.enc_params.L1_sep << std::endl;
+    std::cout << "\tBlock parameters : xblen="  << enc_ctx.enc_params.xblen;
+    std::cout << " yblen=" << enc_ctx.enc_params.yblen;
+    std::cout << " xbsep=" << enc_ctx.enc_params.xbsep;
+    std::cout << " ybsep=" << enc_ctx.enc_params.ybsep << std::endl;
+}
 
 int main (int argc, char* argv[])
 {
     /*********************************************************************************/
             /**********  command line parameter parsing*********/
 
-         /********** create params object to handle command line parameter parsing*********/
-    //To do: put parsing in a different function/constructor.
+    // An array indicating whether a parameter has been parsed
+    bool* parsed = new bool[argc];
+
+    // Program name has been parsed
+    parsed[0] = true;    
+
+    // No other parameters 
+    for (int i=1 ; i<argc ; ++i )
+        parsed[i] = false;
+
 
     // the variables we'll read parameters into
     dirac_encoder_context_t enc_ctx;
@@ -386,7 +440,7 @@ int main (int argc, char* argv[])
     int start_pos = 0;
     int end_pos = -1;
     bool verbose = false;
-    bool nolocal = false;
+    bool nolocal = true;
 
     memset (&enc_ctx, 0, sizeof(dirac_encoder_context_t));
     if (argc<3)//need at least 3 arguments - the program name, an input and 
@@ -404,18 +458,22 @@ int main (int argc, char* argv[])
         if ( strcmp (argv[i], "-CIF") == 0 )
         {
             preset = CIF;
+            parsed[i] = true;
         }
         else if ( strcmp (argv[i], "-HD720") == 0 )
         {
             preset = HD720;
+            parsed[i] = true;
         }
         else if ( strcmp (argv[i], "-HD1080") == 0 )
         {
             preset = HD1080;
+            parsed[i] = true;
         }
         else if ( strcmp (argv[i], "-SD576") == 0 )
         {
             preset = SD576;
+            parsed[i] = true;
         }
     }
 
@@ -427,46 +485,57 @@ int main (int argc, char* argv[])
     {
         if ( strcmp(argv[i], "-width") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.seq_params.width =  
                 strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
 
         if ( strcmp(argv[i], "-height") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.seq_params.height =  
                 strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         
         if ( strcmp(argv[i], "-cformat") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.seq_params.chroma =  
                 (ChromaFormat)strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
 
         if ( strcmp(argv[i], "-fr") == 0 )
         {
+            parsed[i] = true;
             i++;
             if(strncmp(argv[i], "59.94", 5)==0)
             {
-                 enc_ctx.seq_params.frame_rate.numerator=60000;
-                 enc_ctx.seq_params.frame_rate.denominator=1001;
+                parsed[i] = true;
+                enc_ctx.seq_params.frame_rate.numerator=60000;
+                enc_ctx.seq_params.frame_rate.denominator=1001;
             }
             else if(strncmp(argv[i], "23.98", 5)==0)
             {
+                parsed[i] = true;
                 enc_ctx.seq_params.frame_rate.numerator=24000; 
                 enc_ctx.seq_params.frame_rate.denominator=1001;
             }
             else if(strncmp(argv[i], "29.97", 5)==0)
             {
+                parsed[i] = true;
                 enc_ctx.seq_params.frame_rate.numerator=30000;
                 enc_ctx.seq_params.frame_rate.denominator=1001;
             }
             //test for decimal format
             else if(strcspn(argv[i], ".")!=strlen(argv[i]))
             {
+                parsed[i] = true;
                 // look for whole number
                 char* num_token = strtok(argv[i], ".");
                 int whole = strtoul(num_token,NULL,10);
@@ -490,6 +559,7 @@ int main (int argc, char* argv[])
             }
             else 
             {
+                parsed[i] = true;
                 // assume e/d format
                 char* token = strtok(argv[i], "/");
                 enc_ctx.seq_params.frame_rate.numerator =  
@@ -503,68 +573,95 @@ int main (int argc, char* argv[])
 
         if ( strcmp(argv[i], "-qf") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.enc_params.qf =  atof(argv[i]);
+            parsed[i] = true;
+        }
+        else if ( strcmp(argv[i], "-lossless") == 0 )
+        {
+            parsed[i] = true;
+            enc_ctx.enc_params.lossless = true;
         }
         else if ( strcmp(argv[i], "-L1_sep") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.enc_params.L1_sep =  
                 strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-num_L1") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.enc_params.num_L1 = 
                 strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-xblen") == 0 )
         {
+            parsed[i] = true;
             i++;
             enc_ctx.enc_params.xblen = 
                 strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-yblen") == 0 )
         {
+            parsed[i] = true;
             i++;
              enc_ctx.enc_params.yblen = 
                  strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-xbsep") == 0 )
         {
+            parsed[i] = true;
             i++;
              enc_ctx.enc_params.xbsep = 
                  strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-ybsep") == 0 )
         {
+            parsed[i] = true;
             i++;
              enc_ctx.enc_params.ybsep = 
                  strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-cpd") == 0 )
         {
+            parsed[i] = true;
             i++;
              enc_ctx.enc_params.cpd = 
                  strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-verbose") == 0 )
         {
+            parsed[i] = true;
             verbose = true;
         }
-        else if ( strcmp(argv[i], "-nolocal") == 0 )
+        else if ( strcmp(argv[i], "-local") == 0 )
         {
-            nolocal = true;
+            parsed[i] = true;
+            nolocal = false;
         }
         else if ( strcmp(argv[i], "-start") == 0 )
         {
+            parsed[i] = true;
             i++;
             start_pos = strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         else if ( strcmp(argv[i], "-stop") == 0 )
         {
+            parsed[i] = true;
             i++;
             end_pos = strtoul(argv[i],NULL,10);
+            parsed[i] = true;
         }
         i++;
     }//opt
@@ -578,6 +675,8 @@ int main (int argc, char* argv[])
 
     input=argv[argc-2];
     output=argv[argc-1];
+    parsed[argc-2] = true;
+    parsed[argc-1] = true;
 
     //check we have real inputs
     if ((input.length() == 0) || (output.length() ==0))
@@ -586,6 +685,29 @@ int main (int argc, char* argv[])
         exit(1);
     }
 
+    // check we have parsed everything
+    bool all_parsed = true;
+    for (int i=0 ; i<argc ; ++i)
+    {
+        if ( !parsed[i] )
+        {
+            all_parsed = false;
+            std::cerr<<std::endl<<"Unknown option "<<argv[i];
+        }
+    }
+    if ( !all_parsed )
+    {
+        display_help();
+        exit(1);
+    }
+
+    if (strcmp (input.c_str(), output.c_str()) == 0)
+    {
+        std::cerr << "Input and output file names must be different" << std::endl;
+        exit(1);
+    }
+
+    display_codec_params(enc_ctx);
     bit_name = output + ".drc";
         
 
@@ -612,11 +734,16 @@ int main (int argc, char* argv[])
 
     // open the decoded ouput file
     std::string output_name_yuv = output + ".yuv";
-    std::ofstream outyuv(output_name_yuv.c_str(),std::ios::out | std::ios::binary);
+    std::string output_name_imt = output + ".imt";
+    std::ofstream *outyuv = NULL, *outimt = NULL;
+    
+    if (nolocal == false)
+    {
+        outyuv = new std::ofstream(output_name_yuv.c_str(),std::ios::out | std::ios::binary);
 
       // open the diagnostics ouput file
-    std::string output_name_imt = output + ".imt";
-    std::ofstream outimt(output_name_imt.c_str(),std::ios::out | std::ios::binary);
+        outimt = new std::ofstream(output_name_imt.c_str(),std::ios::out | std::ios::binary);
+    }
 
    /********************************************************************/
     //do the work!!
@@ -648,8 +775,6 @@ int main (int argc, char* argv[])
         
     encoder = dirac_encoder_init( &enc_ctx, verbose );
 
-    WriteSequenceHeader ( outimt, encoder );
-
     if (!encoder)
     {
         std::cerr << "Unrecoverable Error: dirac_encoder_init failed. "
@@ -657,8 +782,15 @@ int main (int argc, char* argv[])
         return EXIT_FAILURE;
     }
 
+    if (outimt)
+        WriteSequenceHeader ( *outimt, encoder );
+
     int frames_written = 0;
     dirac_encoder_state_t state;
+
+    clock_t start_t, stop_t;
+    start_t = clock();
+
     do 
     {
         if (ReadPicData( ip_pic_ptr, frame_buf, frame_size ) == true)
@@ -700,24 +832,33 @@ int main (int argc, char* argv[])
                 break;
             }
 
-            WritePicData (outyuv, encoder);
-            WriteDiagnosticsData (outimt, encoder);
+            WritePicData (*outyuv, encoder);
+            WriteDiagnosticsData (*outimt, encoder);
 
         } while (state == ENC_STATE_AVAIL);
 
     } while (frames_written <= (end_pos - start_pos));
 
+    stop_t = clock();
+
     encoder->enc_buf.buffer = video_buf;
     encoder->enc_buf.size = VIDEO_BUFFER_SIZE;
     if (dirac_encoder_end_sequence( encoder ) > 0)
     {
         outfile.write((char *)encoder->enc_buf.buffer, 
                       encoder->enc_buf.size);
-           std::cout << "The resulting bit-rate at "
-                  << (double)encoder->enc_ctx.seq_params.frame_rate.numerator/
-                      encoder->enc_ctx.seq_params.frame_rate.denominator
-                  << "Hz is " << encoder->enc_seqstats.bit_rate 
-                  << " bits/sec." << std::endl;
+
+        if ( verbose )           
+            std::cout << "The resulting bit-rate at "
+                      << (double)encoder->enc_ctx.seq_params.frame_rate.numerator/
+                          encoder->enc_ctx.seq_params.frame_rate.denominator
+                      << "Hz is " << encoder->enc_seqstats.bit_rate 
+                      << " bits/sec." << std::endl;
+
+        if ( verbose )
+            std::cout<<"Time per frame: "<<
+                    (double)(stop_t-start_t)/(double)(CLOCKS_PER_SEC*frames_written);
+            std::cout<<std::endl<<std::endl;
     }
 
    
@@ -728,11 +869,18 @@ int main (int argc, char* argv[])
      // close the bitstream file
     outfile.close();
      // close the decoded output file
-    outyuv.close();
+    if (outyuv)
+    {
+        outyuv->close();
+        delete outyuv;
+    }
    
      // close the decoded output header file
-    outimt.close();
-
+     if (outimt)
+     {
+        outimt->close();
+        delete outimt;
+     }
     // close the pic data file
     ip_pic_ptr.close();
 
@@ -741,4 +889,6 @@ int main (int argc, char* argv[])
     delete [] frame_buf;
         return EXIT_SUCCESS;
 
+    delete[] parsed;
+
 }
