@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: band_codec.h,v 1.13 2005/04/06 14:31:10 tjdwave Exp $ $Name: Dirac_0_5_2 $
+* $Id: band_codec.h,v 1.15 2005/12/02 10:16:52 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,9 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author), Scott R Ladd
+* Contributor(s): Thomas Davies (Original Author),
+*                 Scott R Ladd,
+*                 Steve Bearcroft
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -92,7 +94,10 @@ namespace dirac
         void DecodeVal(PicArray& out_data , const int xpos , const int ypos );
 
         //! Set a code block area to a given value
-        void SetToVal( const CodeBlock& code_block , PicArray& pic_data , const ValueType val);
+        inline void SetToVal( const CodeBlock& code_block , PicArray& pic_data , const ValueType val);
+
+        //! Set all block values to 0
+        inline void ClearBlock( const CodeBlock& code_block , PicArray& pic_data);
 
     private:
         //functions
@@ -105,8 +110,29 @@ namespace dirac
         virtual void DecodeCoeffBlock(const CodeBlock& code_block , PicArray& out_data);
 
         void ResetAll();
-    
-        int ChooseContext( const int bin_number) const;
+
+        // structure used to select the contexts used to encode / decode a symbol
+        typedef struct 
+        {
+            int firstContext;
+            int secondContext;
+            int endContext;
+        } ContextTriple;
+
+        typedef enum ContextTripleAliases
+        {
+            Z_z_CTXT,
+            Z_nz_CTXT,
+            NZ_z_CTXT,
+            NZ_b_CTXT,
+            NZ_a_CTXT,
+            lastCTXTAlias
+        } ;
+
+        static const ContextTriple m_context_triples[lastCTXTAlias];
+
+        const ContextTriple& ChooseContexts() const;
+
         int ChooseSignContext(const PicArray& data , const int xpos , const int ypos ) const;
 
         //! Private, bodyless copy constructor: class should not be copied
