@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: common.cpp,v 1.14 2005/05/19 10:21:30 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: common.cpp,v 1.19 2005/09/27 08:50:36 tjdwave Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -200,7 +200,8 @@ CodecParams::CodecParams():
     m_topfieldfirst(false),
     m_lbparams(3),
     m_cbparams(3),
-    m_mv_precision(2)
+    m_mv_precision(2),
+    m_wlt_filter(APPROX97)
 {}
 
 void CodecParams::SetBlockSizes(const OLBParams& olbparams , const ChromaFormat cformat)
@@ -305,7 +306,21 @@ void CodecParams::SetBlockSizes(const OLBParams& olbparams , const ChromaFormat
     m_cbparams[0].SetXbsep( m_cbparams[1].Xbsep()*2 );
     m_cbparams[0].SetXblen( m_cbparams[1].Xblen() + m_cbparams[1].Xbsep() );
     m_cbparams[0].SetYbsep( m_cbparams[1].Ybsep()*2 );
-    m_cbparams[0].SetYblen( m_cbparams[1].Yblen() + m_cbparams[1].Xbsep() );        
+    m_cbparams[0].SetYblen( m_cbparams[1].Yblen() + m_cbparams[1].Xbsep() );
+
+    if ( m_lbparams[2].Xbsep()!=olbparams.Xbsep() ||
+         m_lbparams[2].Ybsep()!=olbparams.Ybsep() ||
+         m_lbparams[2].Xblen()!=olbparams.Xblen() ||
+         m_lbparams[2].Yblen()!=olbparams.Yblen() )
+    {
+        std::cerr<<std::endl<<"WARNING: block parameters are inconsistent or ";
+        std::cerr<<"incompatible with chroma format.";
+        std::cerr<<std::endl<<"Instead, using:";
+        std::cerr<<" xblen="<<m_lbparams[2].Xblen();
+        std::cerr<<" yblen="<<m_lbparams[2].Yblen();
+        std::cerr<<" xbsep="<<m_lbparams[2].Xbsep();
+        std::cerr<<" ybsep="<<m_lbparams[2].Ybsep();
+    }
     
 }
 
@@ -314,7 +329,10 @@ void CodecParams::SetBlockSizes(const OLBParams& olbparams , const ChromaFormat
 //Default constructor    
 EncoderParams::EncoderParams():
     CodecParams(),
-    m_qf(5.0),
+
+    m_loc_decode(true),
+    m_lossless(false),
+    m_qf(7.0),
     m_num_L1(0),
     m_L1_sep(0),
     m_ufactor(1.0),
