@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: common.h,v 1.22 2005/05/19 10:21:30 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: common.h,v 1.26 2005/09/27 08:50:36 tjdwave Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -591,6 +591,9 @@ namespace dirac
 
         //! Return the number of accuracy bits used for motion vectors
         int MVPrecision() const { return m_mv_precision; }
+
+        //! Return the wavelet filter currently being used for frame (de)coding
+        WltFilter TransformFilter() const { return m_wlt_filter; } 
              
         // ... and Sets
         //! Set how many MBs there are horizontally
@@ -631,6 +634,9 @@ namespace dirac
             m_mv_precision = p;
         }
 
+        //! Return the wavelet filter currently being used for frame (de)coding
+        void SetTransformFilter(const WltFilter wf) { m_wlt_filter=wf; } 
+
     private:
         
         //! The number of macroblocks horizontally
@@ -665,6 +671,9 @@ namespace dirac
 
         //! The precision of motion vectors (number of accuracy bits eg 1=half-pel accuracy) 
         int m_mv_precision;
+
+        //! The wavelet filter being used
+        WltFilter m_wlt_filter;
     };
 
     //! Parameters for the encoding process
@@ -686,6 +695,12 @@ namespace dirac
         
          // Gets ...
 
+        //! Returns a flag indicating that we're doing local decoding
+        bool LocalDecode() const {return m_loc_decode;}
+
+        //! Get whether we're doing lossless coding
+        float Lossless() const {return m_lossless;}
+
         //! Get the quality factor
         float Qf() const {return m_qf;}
 
@@ -744,6 +759,12 @@ namespace dirac
         
         // ... and Sets
 
+        //! Sets a flag indicating that we're producing a locally decoded o/p
+        void SetLocalDecode( const bool decode ){m_loc_decode=decode;}
+
+        //! Set whether we're doing lossless coding
+        void SetLossless(const bool l){m_lossless = l;}
+
         //! Set the quality factor
         void SetQf(const float qfac){m_qf=qfac;}
 
@@ -790,14 +811,21 @@ namespace dirac
         void SetBitsOut( SequenceOutputManager* so ){ m_bit_out=so; }
         
     private:
-        //! Quality factor (between 0 and 10)
+
+        //! Flag indicating we're doing local decoding
+        bool m_loc_decode;
+      
+        //! A flag indicating we're doing lossless coding
+        bool m_lossless;
+
+        //! Quality factor
         float m_qf; 
         
         //! Number of L1 frames before next I frame
         int m_num_L1;
         
         //! Separation between L1 frames
-        int m_L1_sep;    
+        int m_L1_sep;   
         
         //! factor for weighting U component quantisation errors
         float m_ufactor;
