@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: mot_comp.cpp,v 1.17 2005/05/19 10:16:08 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: mot_comp.cpp,v 1.22 2005/12/01 14:06:46 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -41,6 +41,9 @@
 
 
 #include <libdirac_common/mot_comp.h>
+#if defined(HAVE_MMX)
+#include <libdirac_common/mot_comp_mmx.h>
+#endif
 #include <libdirac_common/motion.h>
 #include <libdirac_common/frame_buffer.h>
 using namespace dirac;
@@ -100,9 +103,13 @@ MotionCompensator::MotionCompensator( const CodecParams &cp ):
     m_cparams(cp),
     luma_or_chroma(true)
 {
+    m_block_weights = new TwoDArray<ValueType>[9];
+    m_half_block_weights = new TwoDArray<ValueType>[9];
+    m_macro_block_weights = new TwoDArray<ValueType>[9];
+    m_half_macro_block_weights = new TwoDArray<ValueType>[9];
+    m_sub_block_weights = new TwoDArray<ValueType>[9];
+    m_half_sub_block_weights = new TwoDArray<ValueType>[9];
     //Configure weighting blocks for the first time
-    m_block_weights = NULL;
-    m_half_block_weights = NULL;
     ReConfig();
 }
 
@@ -112,6 +119,10 @@ MotionCompensator::~MotionCompensator(){
 //Tidy up the pointers
     delete[] m_block_weights;
     delete[] m_half_block_weights;
+    delete[] m_macro_block_weights;
+    delete[] m_half_macro_block_weights;
+    delete[] m_sub_block_weights;
+    delete[] m_half_sub_block_weights;
 }
 
 //Called to perform motion compensated addition/subtraction on an entire frame.
@@ -142,7 +153,7 @@ void MotionCompensator::CompensateFrame( const AddOrSub direction ,
                  ref2_idx = refs[0];
 
              const Frame& ref1frame = my_buffer.GetFrame(ref1_idx);
-             const Frame& ref2frame = my_buffer.GetFrame(ref2_idx);
+             const Frame& ref2frame = refs.size() > 0 ? my_buffer.GetFrame(ref2_idx) : ref1frame;
 
              luma_or_chroma = true;                
              //now do all the components
@@ -172,34 +183,68 @@ void MotionCompensator::ReConfig()
     else
         m_bparams = m_cparams.ChromaBParams(2);
 
-    if(m_block_weights != NULL)
-        delete[] m_block_weights;
+    int blocks_per_mb_row = m_cparams.XNumBlocks()/m_cparams.XNumMB();
+    int blocks_per_sb_row = blocks_per_mb_row>>1;
+    int mb_xlen = m_bparams.Xblen()*blocks_per_mb_row - (m_bparams.Xblen()-m_bparams.Xbsep())*(blocks_per_mb_row-1);
+    int mb_ylen = m_bparams.Yblen();
+    int mb_xsep = mb_xlen - (m_bparams.Xblen()-m_bparams.Xbsep());
+    int mb_ysep = m_bparams.Ybsep();
+    int sb_xlen = m_bparams.Xblen()*blocks_per_sb_row - (m_bparams.Xblen()-m_bparams.Xbsep())*(blocks_per_sb_row-1);
+    int sb_ylen = m_bparams.Yblen();
+    int sb_xsep = sb_xlen - (m_bparams.Xblen() - m_bparams.Xbsep());
+    int sb_ysep = m_bparams.Ybsep();
 
-    if(m_half_block_weights != NULL)
-        delete[] m_half_block_weights;
-
-    // Create new weights array.
-    m_block_weights = new TwoDArray<CalcValueType>[9];
-    m_half_block_weights = new TwoDArray<CalcValueType>[9];
     for(int i = 0; i < 9; i++)
     {
         m_block_weights[i].Resize(  m_bparams.Yblen() , m_bparams.Xblen() );
         m_half_block_weights[i].Resize(  m_bparams.Yblen() , m_bparams.Xblen() );
+        m_macro_block_weights[i].Resize(  mb_ylen , mb_xlen );
+        m_half_macro_block_weights[i].Resize(  mb_ylen , mb_xlen );
+        m_sub_block_weights[i].Resize(  sb_ylen , sb_xlen );
+        m_half_sub_block_weights[i].Resize(  sb_ylen , sb_xlen );
     }
     // We can create all nine weighting blocks by calculating values
     // for four blocks and mirroring them to generate the others.
-    CreateBlock( m_bparams , false , false , m_half_block_weights[0] );
-    CreateBlock( m_bparams , false , true , m_half_block_weights[3] );
-    CreateBlock( m_bparams , true , false , m_half_block_weights[1] );
-    CreateBlock( m_bparams , true , true , m_half_block_weights[4] );
+    CreateBlock( m_bparams.Xblen(), m_bparams.Yblen() , m_bparams.Xbsep(), m_bparams.Ybsep(), false , false , m_half_block_weights[0] );
+    CreateBlock( m_bparams.Xblen(), m_bparams.Yblen() , m_bparams.Xbsep(), m_bparams.Ybsep(), false , true , m_half_block_weights[3] );
+    CreateBlock( m_bparams.Xblen(), m_bparams.Yblen() , m_bparams.Xbsep(), m_bparams.Ybsep(), true , false , m_half_block_weights[1] );
+    CreateBlock( m_bparams.Xblen(), m_bparams.Yblen() , m_bparams.Xbsep(), m_bparams.Ybsep(), true , true , m_half_block_weights[4] );
 
     // Note order of flipping is important.    
-    FlipX( m_half_block_weights[3] , m_bparams , m_half_block_weights[5] );
-    FlipX( m_half_block_weights[0] , m_bparams , m_half_block_weights[2] );
-    FlipY( m_half_block_weights[0] , m_bparams , m_half_block_weights[6] );
-    FlipX( m_half_block_weights[6] , m_bparams , m_half_block_weights[8] );
-    FlipY( m_half_block_weights[1] , m_bparams , m_half_block_weights[7] );
+    FlipX( m_half_block_weights[3] , m_bparams.Xblen() , m_bparams.Yblen() , m_half_block_weights[5] );
+    FlipX( m_half_block_weights[0] , m_bparams.Xblen() , m_bparams.Yblen() , m_half_block_weights[2] );
+    FlipY( m_half_block_weights[0] , m_bparams.Xblen() , m_bparams.Yblen() , m_half_block_weights[6] );
+    FlipX( m_half_block_weights[6] , m_bparams.Xblen() , m_bparams.Yblen() , m_half_block_weights[8] );
+    FlipY( m_half_block_weights[1] , m_bparams.Xblen() , m_bparams.Yblen() , m_half_block_weights[7] );
 
+    // We can create all nine weighting blocks by calculating values
+    // for four blocks and mirroring them to generate the others.
+    CreateBlock( mb_xlen, mb_ylen, mb_xsep, mb_ysep , false , false , m_half_macro_block_weights[0] );
+    CreateBlock( mb_xlen, mb_ylen, mb_xsep, mb_ysep , false , true , m_half_macro_block_weights[3] );
+    CreateBlock( mb_xlen, mb_ylen, mb_xsep, mb_ysep , true , false , m_half_macro_block_weights[1] );
+    CreateBlock( mb_xlen, mb_ylen, mb_xsep, mb_ysep , true , true , m_half_macro_block_weights[4] );
+
+    // Note order of flipping is important.    
+    FlipX( m_half_macro_block_weights[3] , mb_xlen, mb_ylen , m_half_macro_block_weights[5] );
+    FlipX( m_half_macro_block_weights[0] , mb_xlen, mb_ylen , m_half_macro_block_weights[2] );
+    FlipY( m_half_macro_block_weights[0] , mb_xlen, mb_ylen , m_half_macro_block_weights[6] );
+    FlipX( m_half_macro_block_weights[6] , mb_xlen, mb_ylen , m_half_macro_block_weights[8] );
+    FlipY( m_half_macro_block_weights[1] , mb_xlen, mb_ylen , m_half_macro_block_weights[7] );
+    
+    // We can create all nine weighting blocks by calculating values
+    // for four blocks and mirroring them to generate the others.
+    CreateBlock( sb_xlen, sb_ylen, sb_xsep, sb_ysep , false , false , m_half_sub_block_weights[0] );
+    CreateBlock( sb_xlen, sb_ylen, sb_xsep, sb_ysep , false , true , m_half_sub_block_weights[3] );
+    CreateBlock( sb_xlen, sb_ylen, sb_xsep, sb_ysep , true , false , m_half_sub_block_weights[1] );
+    CreateBlock( sb_xlen, sb_ylen, sb_xsep, sb_ysep , true , true , m_half_sub_block_weights[4] );
+
+    // Note order of flipping is important.    
+    FlipX( m_half_sub_block_weights[3] , sb_xlen, sb_ylen , m_half_sub_block_weights[5] );
+    FlipX( m_half_sub_block_weights[0] , sb_xlen, sb_ylen , m_half_sub_block_weights[2] );
+    FlipY( m_half_sub_block_weights[0] , sb_xlen, sb_ylen , m_half_sub_block_weights[6] );
+    FlipX( m_half_sub_block_weights[6] , sb_xlen, sb_ylen , m_half_sub_block_weights[8] );
+    FlipY( m_half_sub_block_weights[1] , sb_xlen, sb_ylen , m_half_sub_block_weights[7] );
+   
     for( int k = 0; k < 9; k++)
     {
         for ( int j =m_half_block_weights[k].FirstY(); j <= m_half_block_weights[k].LastY(); j++)
@@ -210,6 +255,26 @@ void MotionCompensator::ReConfig()
             }// i
         }// j
     }// k
+    for( int k = 0; k < 9; k++)
+    {
+        for ( int j =m_half_macro_block_weights[k].FirstY(); j <= m_half_macro_block_weights[k].LastY(); j++)
+        {
+            for ( int i =m_half_macro_block_weights[k].FirstX(); i <= m_half_macro_block_weights[k].LastX(); i++)
+            {
+                m_macro_block_weights[k][j][i] = m_half_macro_block_weights[k][j][i] << 1;
+            }// i
+        }// j
+    }// k
+    for( int k = 0; k < 9; k++)
+    {
+        for ( int j =m_half_sub_block_weights[k].FirstY(); j <= m_half_sub_block_weights[k].LastY(); j++)
+        {
+            for ( int i =m_half_sub_block_weights[k].FirstX(); i <= m_half_sub_block_weights[k].LastX(); i++)
+            {
+                m_sub_block_weights[k][j][i] = m_half_sub_block_weights[k][j][i] << 1;
+            }// i
+        }// j
+    }// k
 }
 
 void MotionCompensator::CompensateComponent( Frame& picframe , 
@@ -221,33 +286,36 @@ void MotionCompensator::CompensateComponent( Frame& picframe ,
     // Set up references to pictures and references
     PicArray& pic_data_out = picframe.Data( cs );
 
+    // Size of frame component being motion compensated
+    ImageCoords orig_pic_size(pic_data_out.LengthX(), pic_data_out.LengthY());
+
     const PicArray& ref1up = ref1frame.UpData( cs );
     const PicArray& ref2up = ref2frame.UpData( cs );
 
-    // Set up another picture which will contain the MC data, which
+    // Set up a row of blocks which will contain the MC data, which
     // we'll add or subtract to pic_data_out
-    TwoDArray<CalcValueType> pic_data(pic_data_out.LengthY(), pic_data_out.LengthX() );
+    TwoDArray<CalcValueType> pic_data(m_bparams.Yblen(), pic_data_out.LengthX(), 0 );
 
     // Factors to compensate for subsampling of chroma
-    int xscale_factor = 1;
-    int yscale_factor = 1;
+    int xscale_shift = 0;
+    int yscale_shift = 0;
 
     if ( cs != Y_COMP )
     {
         if (m_cformat == format420)
         {
-            xscale_factor = 2;
-            yscale_factor = 2;
+            xscale_shift = 1;
+            yscale_shift = 1;
         }
         else if (m_cformat == format422)
         {
-            xscale_factor = 2;
-            yscale_factor = 1;
+            xscale_shift = 1;
+            yscale_shift = 0;
         }
         else if (m_cformat == format411)
         {
-            xscale_factor = 4;
-            yscale_factor = 1;
+            xscale_shift = 2;
+            yscale_shift = 0;
         }
 
     } 
@@ -280,23 +348,22 @@ void MotionCompensator::CompensateComponent( Frame& picframe ,
     //and add the compensated pixels to the image pointed to by pic_data.
     size_t wgt_idx;
 
-    int first_uncleared_row = 0;
-
     //Loop over all the block rows
     pos.y = -m_bparams.Yoffset();
+    int save_from_row = m_bparams.Ybsep()-m_bparams.Yoffset();
+
+    const int x_end_data = pic_data_out.FirstX() + std::min(pic_data_out.LengthX(), m_cparams.XNumBlocks()*m_bparams.Xbsep() );
+    const int y_end_data = pic_data_out.FirstY() + std::min(pic_data_out.LengthY(), m_cparams.YNumBlocks()*m_bparams.Ybsep() );
+
+    const int blocks_per_mb_row = m_cparams.XNumBlocks()/m_cparams.XNumMB();
+    const int blocks_per_sb_row = blocks_per_mb_row>>1;
+
     for(int yblock = 0; yblock < m_cparams.YNumBlocks(); ++yblock)
     {
-
-        int clear_to_row = std::min( pos.y+m_bparams.Yblen(), pic_data.LengthY() );
-        if( clear_to_row > first_uncleared_row )
-        {
-            memset( pic_data[first_uncleared_row], 0, (clear_to_row-first_uncleared_row)*pic_data.LengthX()*sizeof(CalcValueType) );
-            first_uncleared_row = clear_to_row;
-        }
-
         pos.x = -m_bparams.Xoffset();
+        int xincr, xb_incr = 0;
         //loop over all the blocks in a row
-        for(int xblock = 0 ; xblock < m_cparams.XNumBlocks(); ++xblock)
+        for(int xblock = 0 ; xblock < m_cparams.XNumBlocks(); xblock+=xb_incr)
         {
 
             //Decide which weights to use.
@@ -330,115 +397,181 @@ void MotionCompensator::CompensateComponent( Frame& picframe ,
 
             block_mode = mv_data.Mode()[yblock][xblock];
 
+            TwoDArray<ValueType> *wt, *hwt;
+            int split_mode =  mv_data.MBSplit()[yblock/blocks_per_mb_row][xblock/blocks_per_mb_row];
+            if (split_mode == 0) //Block part of a MacroBlock
+            {
+                // boundary conditions for macro block edge
+                if (xblock == m_cparams.XNumBlocks() - blocks_per_mb_row)
+                {
+                    wgt_idx = wgt_idx+1;
+                }
+                wt = &m_macro_block_weights[wgt_idx];
+                hwt = &m_half_macro_block_weights[wgt_idx];
+                xb_incr = blocks_per_mb_row;
+            }
+            else if (split_mode == 1) //Block part of a SubBlock
+            {
+                // boundary conditions for sub block edge
+                if (xblock == m_cparams.XNumBlocks() - blocks_per_sb_row)
+                {
+                    wgt_idx = wgt_idx+1;
+                }
+                wt = &m_sub_block_weights[wgt_idx];
+                hwt = &m_half_sub_block_weights[wgt_idx];
+                xb_incr = blocks_per_sb_row;
+            }
+            else
+            {
+                wt = &m_block_weights[wgt_idx];
+                hwt = &m_half_block_weights[wgt_idx];
+                xb_incr = 1;
+            }
+            xincr = m_bparams.Xbsep() * xb_incr;
+
+
+
             if(block_mode == REF1_ONLY)
             {
                 mv1 = (*mv_array1)[yblock][xblock];
-                mv1.x /= xscale_factor;
-                mv1.y /= yscale_factor;
+                mv1.x >>= xscale_shift;
+                mv1.y >>= yscale_shift;
 
-                CompensateBlock(pic_data, ref1up, mv1, pos, m_block_weights[wgt_idx]);
+                CompensateBlock(pic_data, orig_pic_size, ref1up, mv1, pos, *wt);
             }
             else if (block_mode == REF2_ONLY)
             {                
                 mv2 = (*mv_array2)[yblock][xblock];
-                mv2.x /= xscale_factor;
-                mv2.y /= yscale_factor;
+                mv2.x >>= xscale_shift;
+                mv2.y >>= yscale_shift;
 
-                CompensateBlock(pic_data, ref2up, mv2, pos, m_block_weights[wgt_idx]);
+                CompensateBlock(pic_data, orig_pic_size, ref2up, mv2, pos, *wt);
             }
             else if(block_mode == REF1AND2)
             {
                 mv1 = (*mv_array1)[yblock][xblock];
-                mv1.x /= xscale_factor;
-                mv1.y /= yscale_factor;
+                mv1.x >>= xscale_shift;
+                mv1.y >>= yscale_shift;
 
-                CompensateBlock(pic_data, ref1up, mv1, pos, m_half_block_weights[wgt_idx]);
+                CompensateBlock(pic_data, orig_pic_size, ref1up, mv1, pos, *hwt);
                 mv2 = (*mv_array2)[yblock][xblock];
-                mv2.x /= xscale_factor;
-                mv2.y /= yscale_factor;
+                mv2.x >>= xscale_shift;
+                mv2.y >>= yscale_shift;
 
-                CompensateBlock(pic_data, ref2up, mv2, pos, m_half_block_weights[wgt_idx]);                    
+                CompensateBlock(pic_data, orig_pic_size, ref2up, mv2, pos, *hwt);
             }
             else
             {//we have a DC block.
                 dc = dcarray[yblock][xblock]<<2;// DC is only given 8 bits, 
                                             // so need to shift to get 10-bit data
 
-                DCBlock(pic_data, dc,pos, m_block_weights[wgt_idx]);
+                DCBlock(pic_data, orig_pic_size, dc,pos, *wt);
             }
 
             //Increment the block horizontal position
-            pos.x += m_bparams.Xbsep();
+            pos.x += xincr;
 
         }//xblock
 
+          // Update the pic data
+          // Use only the first Ybsep rows since the remaining rows are
+          // needed for the next row of blocks since we are using overlapped
+          // blocks motion compensation
+          if (m_add_or_sub == SUBTRACT)
+          {
+              int start_y = std::max(pic_data_out.FirstY() , pos.y) ;
+               int end_y = std::min (pic_data_out.FirstY() + pos.y + m_bparams.Ybsep() , y_end_data); 
+               if ((pic_data_out.FirstY() + pos.y +  m_bparams.Yblen()) >= y_end_data)
+               {
+                    end_y = y_end_data;
+               }
+
+             for ( int i = start_y, pos_y = 0; i < end_y; i++, pos_y++)
+             {
+                 CalcValueType *pic_row = pic_data[pos_y];
+                 ValueType *out_row = pic_data_out[i];
+
+                 for ( int j =pic_data_out.FirstX(); j < x_end_data; ++j)
+                 {
+                     out_row[j] -= static_cast<ValueType>( (pic_row[j] + 1024) >> 11 );
+                 }
+
+                 // Okay, we've done all the actual blocks. Now if the picture is further padded
+                 // we need to set the padded values to zero beyond the last block in the row,
+                 // for all the picture lines in the block row. Need only do this when we're
+                 // subtracting.
+
+                 for (int j=( m_cparams.XNumBlocks()*m_bparams.Xbsep() ); j<pic_data_out.LengthX() ; ++j )
+                 {
+                     out_row[pic_data_out.FirstX()+j] = 0;
+                 }
+             }
+          }
+          else // (m_add_or_sub == ADD)
+          {
+              int start_y = std::max(pic_data_out.FirstY() , pos.y) ;
+              int end_y = std::min (pic_data_out.FirstY() + pos.y + m_bparams.Ybsep() , pic_data_out.FirstY() + pic_data_out.LengthY()); 
+              //if (pos.y +  m_bparams.Yblen() >= pic_data_out.LengthY())
+              if (yblock == (m_cparams.YNumBlocks() - 1))
+              {
+                   end_y += (m_bparams.Yblen()-m_bparams.Ybsep());
+                   if (end_y > pic_data_out.LengthY())
+                        end_y = pic_data_out.LengthY();
+              }
+#if defined (HAVE_MMX)
+             CompensateComponentAddAndShift_mmx (start_y, end_y,
+                                                 pic_data, pic_data_out);
+#else
+             for ( int i = start_y, pic_y = 0; i < end_y; i++, pic_y++)
+             {
+                 CalcValueType *pic_row = pic_data[pic_y];
+                 ValueType *out_row = pic_data_out[i];
+
+                 for ( int j =pic_data_out.FirstX(); j <= pic_data_out.LastX(); j++)
+                 {
+                     out_row[j] += static_cast<ValueType>( (pic_row[j] + 1024) >> 11 ); 
+                 }
+             }
+#endif
+        }
         //Increment the block vertical position
         pos.y += m_bparams.Ybsep();
 
+        // Copy the rows required to motion compensate the next row of block.
+        // This is usually Yblen-Ybsep rows.
+        memmove (pic_data[0], pic_data[save_from_row], (m_bparams.Yblen() - save_from_row)*pic_data.LengthX()*sizeof(CalcValueType));
+        memset( pic_data[m_bparams.Yblen() - save_from_row], 0, save_from_row*pic_data.LengthX()*sizeof(CalcValueType) );
+        save_from_row = m_bparams.Ybsep();
+
     }//yblock
 
     if ( m_add_or_sub == SUBTRACT)
     {
-        int x_end_data = std::min(pic_data.LastX(), m_cparams.XNumBlocks()*m_bparams.Xbsep() );
-        int y_end_data = std::min(pic_data.LastY(), m_cparams.YNumBlocks()*m_bparams.Ybsep() );
-
-        for ( int i =pic_data.FirstY(); i <= y_end_data; i++)
-        {
-            CalcValueType *pic_row = pic_data[i];
-            ValueType *out_row = pic_data_out[i];
-
-            for ( int j =pic_data.FirstX(); j <= x_end_data; ++j)
-            {
-                out_row[j] -= static_cast<ValueType>( (pic_row[j] + 1024) >> 11 );
-            }
-
-            // Okay, we've done all the actual blocks. Now if the picture is further padded
-            // we need to set the padded values to zero beyond the last block in the row,
-            // for all the picture lines in the block row. Need only do this when we're
-            // subtracting.
-
-            for (int j=( m_cparams.XNumBlocks()*m_bparams.Xbsep() ); j<pic_data.LengthX() ; ++j )
-            {
-                out_row[j] = 0;
-            }
-        }
         // Finally, now we've done all the blocks, we must set all padded lines below 
         // the last row equal to 0, if we're subtracting
-        for ( int y=m_cparams.YNumBlocks()*m_bparams.Ybsep() ; y<pic_data.LengthY() ; ++y )
+        for ( int y=m_cparams.YNumBlocks()*m_bparams.Ybsep() ; y<pic_data_out.LengthY() ; ++y )
         {
             ValueType *out_row = pic_data_out[y];
-            for ( int x=0 ; x<pic_data.LengthX() ; ++x )
+            for ( int x=0 ; x<pic_data_out.LengthX() ; ++x )
             {
                 out_row[x] = 0;
             }
 
         }
     }
-    else
-    {
-        for ( int i =pic_data.FirstY(); i <= pic_data.LastY(); i++)
-        {
-            CalcValueType *pic_row = pic_data[i];
-            ValueType *out_row = pic_data_out[i];
-
-            for ( int j =pic_data.FirstX(); j <= pic_data.LastX(); j++)
-            {
-                out_row[j] += static_cast<ValueType>( (pic_row[j] + 1024) >> 11 ); 
-            }
-        }
-    }
 }
 
 void MotionCompensator::DCBlock( TwoDArray<CalcValueType> &pic_data ,
+                                            const ImageCoords& orig_pic_size ,
                                             const ValueType dc , 
                                             const ImageCoords& pos ,
-                                            const TwoDArray<CalcValueType>& wt_array)
+                                            const TwoDArray<ValueType>& wt_array)
 {
 
     //Coordinates in the image being written to.
     const ImageCoords start_pos( std::max(0 , pos.x) , std::max(0 , pos.y) );
-    const ImageCoords end_pos( std::min(pos.x + m_bparams.Xblen() , pic_data.LengthX() ) , 
-                               std::min(pos.y + m_bparams.Yblen() , pic_data.LengthY() ) );
+    const ImageCoords end_pos( std::min(pos.x + wt_array.LengthX() , orig_pic_size.x ) , 
+                               std::min(pos.y + wt_array.LengthY() , orig_pic_size.y ) );
 
     //The difference between the desired start point
     //pos and the actual start point start_pos.
@@ -446,11 +579,19 @@ void MotionCompensator::DCBlock( TwoDArray<CalcValueType> &pic_data ,
 
     //Quick process where we can just copy from the double size image.
 
-    for(int c = start_pos.y, wY = diff.y; c < end_pos.y; ++c, ++wY)
+    CalcValueType *pic_curr = &pic_data[0][start_pos.x]; 
+    ValueType *wt_curr = &wt_array[diff.y][diff.x]; 
+    const int block_width = end_pos.x - start_pos.x;
+    //- go down a row and back up
+    const int pic_next (pic_data.LengthX() - block_width );
+    //- go down a row and back up
+    const int wt_next( wt_array.LengthX() - block_width );
+
+    for(int y = end_pos.y-start_pos.y; y > 0; --y, pic_curr+=pic_next, wt_curr+=wt_next)
     {
-        for(int l = start_pos.x, wX = diff.x; l < end_pos.x; ++l, ++wX)
+        for(int x=block_width; x > 0; --x, ++pic_curr, ++wt_curr)
         {
-            pic_data[c][l] += dc * wt_array[wY][wX];
+            *pic_curr += CalcValueType (dc) * *wt_curr;
         }
     }
 }
@@ -477,119 +618,101 @@ float MotionCompensator::RaisedCosine(float t, float B)
 //   *     *                  *
 //  *       *                  *
 //*           *                  *
-void MotionCompensator::CreateBlock( const OLBParams &bparams , 
-                                                bool FullX , 
-                                                bool FullY , 
-                                                TwoDArray<CalcValueType>& WeightArray)
+void MotionCompensator::CreateBlock( int xblen, int yblen, int xbsep, int ybsep,                                     bool FullX , 
+                                     bool FullY , 
+                                     TwoDArray<ValueType>& WeightArray)
 {
+    const CalcValueType max_val = 32;
+
     // Create temporary arrays
     OneDArray<CalcValueType> HWts( WeightArray.LengthX() );
     OneDArray<CalcValueType> VWts( WeightArray.LengthY() );
 
     // Calculation variables
-    float rolloffX = (float(bparams.Xblen()+1)/float(bparams.Xbsep())) - 1;
-    float rolloffY = (float(bparams.Yblen()+1)/float(bparams.Ybsep())) - 1;
+    float rolloffX = (float(xblen+1)/float(xbsep)) - 1;
+    float rolloffY = (float(yblen+1)/float(ybsep)) - 1;
     float val;
 
     // Window in the x direction
-    for(int x = 0; x < bparams.Xblen(); ++x)
+    for(int x = 0; x < xblen; ++x)
     {
-        val = (float(x) - (float(bparams.Xblen()-1)/2.0))/float(bparams.Xbsep());
-        HWts[x] = static_cast<CalcValueType>( 32.0 * RaisedCosine(val,rolloffX) );
+        val = (float(x) - (float(xblen-1)/2.0))/float(xbsep);
+        HWts[x] = static_cast<CalcValueType>( float(max_val) * RaisedCosine(val,rolloffX)+0.5 );
         HWts[x] = std::max( HWts[x] , 1 );
-        HWts[x] = std::min( HWts[x] , 32 );
+        HWts[x] = std::min( HWts[x] , max_val );
     }// x
 
     // Window in the y direction
-    for(int y = 0; y < bparams.Yblen(); ++y)
+    for(int y = 0; y < yblen; ++y)
     {
-        val = (float(y) - (float(bparams.Yblen()-1)/2.0))/float(bparams.Ybsep());
-        VWts[y] = static_cast<CalcValueType>( 32.0 * RaisedCosine(val,rolloffY) );
+        val = (float(y) - (float(yblen-1)/2.0))/float(ybsep);
+        VWts[y] = static_cast<CalcValueType>( float(max_val) * RaisedCosine(val,rolloffY)+0.5 );
         VWts[y] = std::max( VWts[y] , 1 );
-        VWts[y] = std::min( VWts[y] , 32 );
+        VWts[y] = std::min( VWts[y] , max_val );
     }// y
 
-    // Rationalise to avoid rounding errors
-    for(int x = HWts.Last(); x > HWts.Last()-bparams.Xoffset(); --x)
-    {
-        if (HWts[x] + HWts[HWts.Last()-(x-bparams.Xbsep())] > 32)
-            HWts[HWts.Last()-(x-bparams.Xbsep())] = 32-HWts[x];
-        
-        else if (HWts[x] + HWts[HWts.Last()-(x-bparams.Xbsep())] < 32)
-            HWts[x] = 32-HWts[HWts.Last()-(x-bparams.Xbsep())];
-    }// x 
-
     // Now reflect or pad, as appropriate
     if (!FullX)
     {
-        for( int x = 0; x < (bparams.Xblen()>>1) ; ++x)
-            HWts[x] = 32;
+        for( int x = 0; x < (xblen>>1) ; ++x)
+            HWts[x] = max_val;
     }
     else
     {
-        for( int x = 0; x < (bparams.Xblen()>>1); ++x)
+        for( int x = 0; x < (xblen>>1); ++x)
             HWts[x] = HWts[HWts.Last()-x];
     }
 
-    // Rationalise to avoid rounding errors
-    for(int y = VWts.Last(); y > VWts.Last()-bparams.Yoffset(); --y)
-    {
-        if (VWts[y] + VWts[VWts.Last()-(y-bparams.Ybsep())] > 32)
-            VWts[VWts.Last()-(y-bparams.Ybsep())] = 32-VWts[y];
-        else if (VWts[y] + VWts[VWts.Last()-(y-bparams.Ybsep())] < 32)
-            VWts[y] = 32-VWts[VWts.Last()-(y-bparams.Ybsep())];
-    }// x 
-
     // Reflect or pad, as appropriate
     if (!FullY)
     {
-        for( int y = 0 ; y < (bparams.Yblen()>>1); ++y)
-            VWts[y] = 32;
+        for( int y = 0 ; y < (yblen>>1); ++y)
+            VWts[y] = max_val;
     }
     else
     {
-        for( int y = 0 ; y < (bparams.Yblen()>>1); ++y)
+        for( int y = 0 ; y < (yblen>>1); ++y)
             VWts[y] = VWts[VWts.Last()-y];
     }
 
-    for(int y = 0; y < bparams.Yblen(); ++y)
+    for(int y = 0; y < yblen; ++y)
     {
-        for(int x = 0; x < bparams.Xblen(); ++x)
+        for(int x = 0; x < xblen; ++x)
         {
             WeightArray[y][x] = VWts[y] * HWts[x];
         }// x
     }// y
-
 }
 
 // Flips the values in an array in the x direction.
-void MotionCompensator::FlipX( const TwoDArray<CalcValueType>& Original , 
-                                          const OLBParams &bparams , 
-                                          TwoDArray<CalcValueType>& Flipped)
+void MotionCompensator::FlipX( const TwoDArray<ValueType>& Original , 
+                                          int xblen, int yblen, 
+                                          TwoDArray<ValueType>& Flipped)
 {
-    for(int x = 0; x < bparams.Xblen(); ++x)
+    for(int y = 0; y < yblen; ++y)
     {
-        for(int y = 0; y < bparams.Yblen(); ++y)
+        for(int x = 0; x < xblen; ++x)
         {
-            Flipped[y][x] = Original[y][(bparams.Xblen()-1) - x];
+            Flipped[y][x] = Original[y][(xblen-1) - x];
         }// y
     }// x
 }
 
 // Flips the values in an array in the y direction.
-void MotionCompensator::FlipY( const TwoDArray<CalcValueType>& Original , 
-                                          const OLBParams &bparams , 
-                                          TwoDArray<CalcValueType>& Flipped)
+void MotionCompensator::FlipY( const TwoDArray<ValueType>& Original , 
+                                          int xblen, int yblen,
+                                          TwoDArray<ValueType>& Flipped)
 {
-    for(int x = 0; x < bparams.Xblen(); ++x)
+    for(int y = 0; y < yblen; ++y)
     {
-        for(int y = 0; y < bparams.Yblen(); ++y)
+        for(int x = 0; x < xblen; ++x)
         {
-            Flipped[y][x] = Original[(bparams.Yblen()-1) - y][x];
+            Flipped[y][x] = Original[(yblen-1) - y][x];
         }// y
     }// x
 }
 
+
 // Concrete Sub-Classes
 // Class that implement the CompensateBlock function based on pixel
 // precision values
@@ -602,15 +725,16 @@ MotionCompensator_Pixel::MotionCompensator_Pixel( const CodecParams &cp ) :
 
 void MotionCompensator_Pixel::CompensateBlock( 
                                     TwoDArray<CalcValueType> &pic_data , 
+                                    const ImageCoords& orig_pic_size , 
                                     const PicArray &refup_data , 
                                     const MVector &mv , 
                                     const ImageCoords& pos , 
-                                    const TwoDArray<CalcValueType>& wt_array )
+                                    const TwoDArray<ValueType>& wt_array)
 {
     //Coordinates in the image being written to.
     const ImageCoords start_pos( std::max(pos.x,0) , std::max(pos.y,0) );
-    const ImageCoords end_pos( std::min( pos.x + m_bparams.Xblen() , pic_data.LengthX() ) , 
-                               std::min( pos.y + m_bparams.Yblen() , pic_data.LengthY() ) );
+    const ImageCoords end_pos( std::min( pos.x + wt_array.LengthX() , orig_pic_size.x ) , 
+                               std::min( pos.y + wt_array.LengthY() , orig_pic_size.y ) );
 
     //The difference between the desired start point
     //pos and the actual start point start_pos.
@@ -636,8 +760,8 @@ void MotionCompensator_Pixel::CompensateBlock(
     else if( ref_start.y + ((end_pos.y - start_pos.y - 1)<<1 ) >= refYlen)
         do_bounds_checking = true;
 
-    CalcValueType *pic_curr = &pic_data[start_pos.y][start_pos.x];
-    CalcValueType *wt_curr = &wt_array[diff.y][diff.x];
+    CalcValueType *pic_curr = &pic_data[0][start_pos.x];
+    ValueType *wt_curr = &wt_array[diff.y][diff.x];
 
     const int block_width = end_pos.x - start_pos.x;
 
@@ -681,15 +805,16 @@ MotionCompensator_HalfPixel::MotionCompensator_HalfPixel( const CodecParams &cp
 {}
 
 void MotionCompensator_HalfPixel::CompensateBlock( TwoDArray<CalcValueType> &pic_data , 
-                                                    const PicArray &refup_data , 
-                                                    const MVector &mv , 
-                                                    const ImageCoords& pos , 
-                                                    const TwoDArray<CalcValueType>& wt_array )
+                                          const ImageCoords& orig_pic_size , 
+                                          const PicArray &refup_data , 
+                                          const MVector &mv , 
+                                          const ImageCoords& pos , 
+                                          const TwoDArray<ValueType>& wt_array)
 {
     //Coordinates in the image being written to.
     const ImageCoords start_pos( std::max(pos.x,0) , std::max(pos.y,0) );
-    const ImageCoords end_pos( std::min( pos.x + m_bparams.Xblen() , pic_data.LengthX() ) , 
-                               std::min( pos.y + m_bparams.Yblen() , pic_data.LengthY() ) );
+    const ImageCoords end_pos( std::min( pos.x + wt_array.LengthX() , orig_pic_size.x) , 
+                               std::min( pos.y + wt_array.LengthY() , orig_pic_size.y ) );
 
     //The difference between the desired start point
     //pos and the actual start point start_pos.
@@ -717,8 +842,8 @@ void MotionCompensator_HalfPixel::CompensateBlock( TwoDArray<CalcValueType> &pic
     else if( ref_start.y + ((end_pos.y - start_pos.y - 1 )<<1 ) >= refYlen)
         do_bounds_checking = true;
 
-    CalcValueType *pic_curr = &pic_data[start_pos.y][start_pos.x];
-    CalcValueType *wt_curr = &wt_array[diff.y][diff.x];
+    CalcValueType *pic_curr = &pic_data[0][start_pos.x];
+    ValueType *wt_curr = &wt_array[diff.y][diff.x];
  
     const int block_width = end_pos.x - start_pos.x;
 
@@ -760,16 +885,18 @@ MotionCompensator_QuarterPixel::MotionCompensator_QuarterPixel( const CodecParam
     MotionCompensator( cp )
 {}
 
+#if !defined (HAVE_MMX)
 void MotionCompensator_QuarterPixel::CompensateBlock( TwoDArray<CalcValueType> &pic_data , 
-                                                       const PicArray &refup_data , 
-                                                       const MVector &mv , 
-                                                       const ImageCoords& pos , 
-                                                       const TwoDArray<CalcValueType>& wt_array )
+                                       const ImageCoords& orig_pic_size , 
+                                       const PicArray &refup_data , 
+                                       const MVector &mv , 
+                                       const ImageCoords& pos , 
+                                       const TwoDArray<ValueType>& wt_array)
 {
     //Coordinates in the image being written to.
     const ImageCoords start_pos( std::max(pos.x,0) , std::max(pos.y,0) );
-    const ImageCoords end_pos( std::min( pos.x + m_bparams.Xblen() , pic_data.LengthX() ) , 
-                               std::min( pos.y + m_bparams.Yblen() , pic_data.LengthY() ) );
+    const ImageCoords end_pos( std::min( pos.x + wt_array.LengthX() , orig_pic_size.x ) , 
+                               std::min( pos.y + wt_array.LengthY() , orig_pic_size.y ) );
 
     //The difference between the desired start point
     //pos and the actual start point start_pos.
@@ -789,8 +916,8 @@ void MotionCompensator_QuarterPixel::CompensateBlock( TwoDArray<CalcValueType> &
     //the reference image.
     const int refXlen = refup_data.LengthX();
     const int refYlen = refup_data.LengthY();
-    CalcValueType *pic_curr = &pic_data[start_pos.y][start_pos.x];
-    CalcValueType *wt_curr = &wt_array[diff.y][diff.x];
+    CalcValueType *pic_curr = &pic_data[0][start_pos.x];
+    ValueType *wt_curr = &wt_array[diff.y][diff.x];
 
     const int block_width = end_pos.x - start_pos.x;
 
@@ -809,7 +936,7 @@ void MotionCompensator_QuarterPixel::CompensateBlock( TwoDArray<CalcValueType> &
         do_bounds_checking = true;
     else if( ref_start.y + ((end_pos.y - start_pos.y)<<1 ) >= refYlen)
         do_bounds_checking = true;
-
+     
     if( !do_bounds_checking )
     {
         ValueType *refup_curr = &refup_data[ref_start.y][ref_start.x];
@@ -877,8 +1004,8 @@ void MotionCompensator_QuarterPixel::CompensateBlock( TwoDArray<CalcValueType> &
                                            rmdr.x * rmdr.y };              //br
 
 
-       for(int c = start_pos.y, wY = diff.y, uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
-           c < end_pos.y; ++c, ++wY, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
+       for(int c = 0, wY = diff.y, uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
+           c < end_pos.y - start_pos.y; ++c, ++wY, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
        {
            for(int l = start_pos.x, wX = diff.x, uX = ref_start.x,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen);
                l < end_pos.x; ++l, ++wX, uX += 2,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen))
@@ -895,6 +1022,7 @@ void MotionCompensator_QuarterPixel::CompensateBlock( TwoDArray<CalcValueType> &
 
     }
 }
+#endif
 
 // Motion Compesation class that provides one eighth-pixel precision 
 // compensation
@@ -903,16 +1031,17 @@ MotionCompensator_EighthPixel::MotionCompensator_EighthPixel( const CodecParams
 {}
 
 void MotionCompensator_EighthPixel::CompensateBlock( TwoDArray<CalcValueType> &pic_data , 
-                                                      const PicArray &refup_data , 
-                                                      const MVector &mv , 
-                                                      const ImageCoords& pos , 
-                                                      const TwoDArray<CalcValueType>& wt_array )
+                                      const ImageCoords& orig_pic_size , 
+                                      const PicArray &refup_data , 
+                                      const MVector &mv , 
+                                      const ImageCoords& pos , 
+                                      const TwoDArray<ValueType>& wt_array)
 {
 
     //Coordinates in the image being written to.
     const ImageCoords start_pos( std::max(pos.x,0) , std::max(pos.y,0) );
-    const ImageCoords end_pos( std::min( pos.x + m_bparams.Xblen() , pic_data.LengthX() ) , 
-                               std::min( pos.y + m_bparams.Yblen() , pic_data.LengthY() ) );
+    const ImageCoords end_pos( std::min( pos.x + wt_array.LengthX() , orig_pic_size.x ) , 
+                               std::min( pos.y + wt_array.LengthY() , orig_pic_size.y ) );
 
     //The difference between the desired start point
     //pos and the actual start point start_pos.
@@ -954,9 +1083,9 @@ void MotionCompensator_EighthPixel::CompensateBlock( TwoDArray<CalcValueType> &p
 
     if( !do_bounds_checking )
     {
-        CalcValueType *pic_curr = &pic_data[start_pos.y][start_pos.x];
+        CalcValueType *pic_curr = &pic_data[0][start_pos.x];
         ValueType *refup_curr = &refup_data[ref_start.y][ref_start.x];
-        CalcValueType *wt_curr = &wt_array[diff.y][diff.x];
+        ValueType *wt_curr = &wt_array[diff.y][diff.x];
  
         const int block_width = end_pos.x - start_pos.x;
 
@@ -1020,8 +1149,8 @@ void MotionCompensator_EighthPixel::CompensateBlock( TwoDArray<CalcValueType> &p
     {
         // We're doing bounds checking because we'll fall off the edge of the reference otherwise.
 
-        for(int c = start_pos.y, wY = diff.y, uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
-            c < end_pos.y; ++c, ++wY, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
+        for(int c = 0, wY = diff.y, uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
+            c < end_pos.y - start_pos.y; ++c, ++wY, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
         {
             for(int l = start_pos.x, wX = diff.x, uX = ref_start.x,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen);
                 l < end_pos.x; ++l, ++wX, uX += 2,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen))
