@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: wavelet_utils.h,v 1.16 2005/05/04 13:35:02 tjdwave Exp $ $Name: Dirac_0_5_2 $
+* $Id: wavelet_utils.h,v 1.17 2005/12/01 14:25:00 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -445,6 +445,10 @@ namespace dirac
             //! Return a correction factor to compensate for non-unity power gain of high-pass filter
             double GetHighFactor() const { return 0.81649658;}
 
+#ifdef HAVE_MMX
+            inline void HorizSynth (int xp, int xl, int ystart, int yend, PicArray &pic_data);
+#endif
+
         };
 
         //! Class to do an approximation to Daubechies (9,7) but with just two lifting steps
