@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: dirac_encoder.cpp,v 1.2 2005/05/20 14:01:26 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: dirac_encoder.cpp,v 1.11 2005/10/17 11:06:29 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -442,14 +442,16 @@ void DiracEncoder::SetEncoderParams (const dirac_encoder_context_t *enc_ctx)
     TEST (enc_ctx != NULL);
     OLBParams bparams(12, 12, 8, 8);
 
+    m_encparams.SetLocalDecode(enc_ctx->decode_flag);
     m_encparams.SetQf(enc_ctx->enc_params.qf);
+    m_encparams.SetLossless(enc_ctx->enc_params.lossless);
     m_encparams.SetL1Sep(enc_ctx->enc_params.L1_sep);
     m_encparams.SetNumL1(enc_ctx->enc_params.num_L1);
     m_encparams.SetCPD(enc_ctx->enc_params.cpd);
-    m_encparams.SetUFactor(3.0f);
-    m_encparams.SetVFactor(1.75f);
+    m_encparams.SetUFactor(1.5f);
+    m_encparams.SetVFactor(0.75f);
     bparams.SetYblen( enc_ctx->enc_params.yblen );
-    bparams.SetXbsep( enc_ctx->enc_params.xblen );
+    bparams.SetXblen( enc_ctx->enc_params.xblen );
     bparams.SetYbsep( enc_ctx->enc_params.ybsep );
     bparams.SetXbsep( enc_ctx->enc_params.xbsep );
 
@@ -756,6 +758,7 @@ static void SetEncoderParameters (dirac_encoder_context_t *enc_ctx, dirac_encode
     dirac_encparams_t &encparams = enc_ctx->enc_params;
 
     encparams.qf = 7.0f;
+    encparams.lossless = false;
 
     switch (preset)
     {
@@ -772,17 +775,17 @@ static void SetEncoderParameters (dirac_encoder_context_t *enc_ctx, dirac_encode
         encparams.L1_sep = 3;
         encparams.num_L1 = 7;
         encparams.cpd = 20.0f;
-        encparams.xblen = 16;
-        encparams.yblen = 16;
-        encparams.xbsep = 10;
-        encparams.ybsep = 12;
+        encparams.xblen = 12;
+        encparams.yblen = 12;
+        encparams.xbsep = 8;
+        encparams.ybsep = 8;
         break;
     case HD1080:
         encparams.L1_sep = 3;
         encparams.num_L1 = 3;
         encparams.cpd = 32.0f;
-        encparams.xblen = 20;
-        encparams.yblen = 20;
+        encparams.xblen = 24;
+        encparams.yblen = 24;
         encparams.xbsep = 16;
         encparams.ybsep = 16;
         break;
@@ -869,19 +872,21 @@ extern DllExport dirac_encoder_state_t
 
     try
     {
-        compressor->CompressNextFrame();
-        if (compressor->GetEncodedData (encoder) < 0)
-            ret_stat = ENC_STATE_INVALID;
-        else
+        if (compressor->CompressNextFrame() != 0)
         {
-            if (encoder->enc_buf.size > 0)
+            if (compressor->GetEncodedData (encoder) < 0)
+                ret_stat = ENC_STATE_INVALID;
+            else
             {
-                ret_stat = ENC_STATE_AVAIL;
-            }
+                if (encoder->enc_buf.size > 0)
+                {
+                    ret_stat = ENC_STATE_AVAIL;
+                }
     
-            if (encoder->enc_ctx.decode_flag)
-                compressor->GetDecodedData(encoder);
+            }
         }
+        if (encoder->enc_ctx.decode_flag)
+            compressor->GetDecodedData(encoder);
     }
     catch (...)
     {
