@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion_comp_test.h,v 1.1 2004/11/26 13:36:49 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: motion_comp_test.h,v 1.2 2005/08/01 16:10:37 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -67,5 +67,10 @@ private:
   MotionCompTest( const MotionCompTest &copy );
   void operator =( const MotionCompTest &copy );
 private:
+  void testZeroMotionComp(int precision);
+  void testAddandSubMotionComp(int precision);
+  void testL2_frame(int precision);
+  void testRef2(int precision);
+  void testRef1and2(int precision);
 };
 #endif
