@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: process_sequence.cpp,v 1.4 2004/11/22 13:39:48 asuraparaju Exp $ $Name: Dirac_0_5_2 $
+* $Id: process_sequence.cpp,v 1.5 2005/08/01 16:11:48 asuraparaju Exp $ $Name: Dirac_0_5_4 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -269,6 +269,7 @@ void ProcessSequence::DoSequence(int start, int stop)
             {
                 // look for frame number
                 m_data_in.ignore(100000, ':');
+                data_next_fnum = -1;
                 m_data_in >> data_next_fnum;
 
                 if (m_data_fnum == data_next_fnum && !m_data_in.eof())
@@ -295,6 +296,11 @@ void ProcessSequence::DoSequence(int start, int stop)
                 exit(EXIT_FAILURE);
             }
         }
+        else
+        {
+            if (data_next_fnum == -1)
+                break;
+        }
     }
 
     // close motion data file
